package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IDepartmentService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.Department;
import cn.estsh.i3plus.pojo.platform.bean.Organize;
import cn.estsh.i3plus.pojo.platform.repository.DepartmentRepository;
import cn.estsh.i3plus.pojo.platform.repository.OrganizeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 部门服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-24 20:47
 * @Modify :
 **/
@Service
public class DepartmentService implements IDepartmentService {
    public static final Logger LOGGER = LoggerFactory.getLogger(DepartmentService.class);

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private OrganizeRepository organizeRDao;

    @Autowired
    private CoreTreeService coreTreeService;

    @Override
    public Department insertDepartment(Department department) {
        LOGGER.info("部门信息 Department department:{}", department);
        // 代码唯一校验
        long count = departmentRDao.findByPropertyCount("departmentCode", department.getDepartmentCode());
        if(count <= 0){

            //冗余父节点信息
            if (department.getParentId() != null  && department.getParentId() > 0) {
                LOGGER.info("部门信息 DEPARTMENT :{}", department.getParentId());
                Department parentPosition = departmentRDao.getById(department.getParentId());
                ValidatorBean.checkNotNull(parentPosition,"上级部门信息不存在");

                department.setRedParentName(parentPosition.getName());
            }

            LOGGER.info("组织信息 ORGANIZE :{}", department.getOrganizeId());
            Organize organize = organizeRDao.getById(department.getOrganizeId());
            ValidatorBean.checkNotNull(organize);

            department.setRedOrganizeName(organize.getName());

            LOGGER.info("部门信息 DEPARTMENT {}",department);
            return departmentRDao.insert(department);
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("部门代码已存在")
                    .setErrorSolution("请重新输入部门代码")
                    .build();
        }
    }

    @Override
    public void deleteDepartmentById(String id) {
        LOGGER.info("部门信息 Department id :{}", id);
        // 查询是否存在子级
        LOGGER.info("部门信息 DEPARTMENT :{}", id);
        if (departmentRDao.findByPropertyCount("parentId",Long.parseLong(id)) > 0) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.WMS.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在相关部门信息无法删除！")
                    .setErrorSolution("请先删除子集信息在操作")
                    .build();
        }

        // 删除部门
        LOGGER.info("部门信息 DEPARTMENT :{}", id);
        departmentRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void deleteDepartmentByIds(String[] ids) {
        LOGGER.info("部门信息 DEPARTMENT ids :{}", ids);
        long positionCount = departmentRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("parentId",ids));
        if (positionCount >= 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在相关部门信息无法删除！")
                    .setErrorSolution("请先删除子集信息在操作")
                    .build();
        }else {
            Long[] array = new Long[ids.length];
            for (int i = 0; i < ids.length; i++) {
                array[i] = Long.parseLong(ids[i]);
            }
            departmentRDao.deleteByIds(array);
        }
    }

    @Override
    public void updateDepartment(Department department) {
        LOGGER.info("部门信息 DEPARTMENT department :{}", department);
        // 唯一校验 CODE 相同 ID 不同 则视为相同
        List<Department> list = departmentRDao.findByProperty("departmentCode", department.getDepartmentCode());
        if(list != null && list.size() > 0){
            for (Department dep : list) {
                if(!dep.getId().equals(department.getId())){
                    throw ImppExceptionBuilder.newInstance()
                            .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                            .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                            .setErrorDetail("部门代码已存在")
                            .setErrorSolution("请重新输入部门代码")
                            .build();
                }
            }
        }

        // 父节点检查
        coreTreeService.doValidatorDepartmentParentId(department);

        // 查询父级部门名称
        if (department.getParentId() != null  && department.getParentId() > 0) {
            LOGGER.info("部门信息 DEPARTMENT :{}", department.getParentId());
            Department parentPosition = departmentRDao.getById(department.getParentId());
            ValidatorBean.checkNotNull(parentPosition,"上级部门信息不存在");

            department.setRedParentName(parentPosition.getName());
        }

        // 查询所属组织名称
        LOGGER.info("组织信息 ORGANIZE organizeId:{}", department.getOrganizeId());
        Organize organize = organizeRDao.getById(department.getOrganizeId());
        ValidatorBean.checkNotNull(organize);

        department.setRedOrganizeName(organize.getName());

        // 新增部门
        LOGGER.info("部门信息 DEPARTMENT department:{}",department);
        departmentRDao.update(department);
        // 修改冗余信息
        // 修改冗余信息
        departmentRDao.updateByProperties("parentId",department.getId(),
                "redParentName",department.getName());
    }

    @Override
    public List<Department> listDepartment() {
        LOGGER.info("部门信息 DEPARTMENT list");
        return  departmentRDao.list();
    }

    @Override
    public ListPager<Department> queryDepartment(Department department, Pager pager) {
        LOGGER.info("部门 Department find  department :{} page:{}", department, pager);

        if (department == null) {
            //不传入实体对象，查询所有
            int count = departmentRDao.listCount();
            pager = PagerHelper.getPager(pager, count);
            return new ListPager(departmentRDao.listPager(pager), pager);
        } else {
            //生成hql查询语句
            String hqlPack = CoreHqlPack.packHqlDepartment(department);
            pager = PagerHelper.getPager(pager, departmentRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(departmentRDao.findByHqlWherePage(hqlPack + department.orderBy(), pager), pager);
        }
    }

    @Override
    public Department getDepartmentById(String id) {
        LOGGER.info("部门信息 DEPARTMENT id:{}",id);
        return departmentRDao.getById(Long.parseLong(id));
    }

    @Override
    public ListPager<Department> findDepartmentByPager(Department department, Pager pager) {
        LOGGER.info("部门信息 DEPARTMENT department :{}", department);
        LOGGER.info("部门信息 DEPARTMENT pager :{}", pager);

        if (department == null) {
            pager = PagerHelper.getPager(pager, departmentRDao.listCount());
            return new ListPager(departmentRDao.listPager(pager),pager);
        } else {
            String hqlPack = CoreHqlPack.packHqlDepartment(department);
            pager = PagerHelper.getPager(pager, departmentRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(departmentRDao.findByHqlWherePage(
                    hqlPack + department.orderBy(),pager),pager);
        }
    }

}
