package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserInfoService;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.*;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 用户信息 服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-11-08 19:44
 * @Modify :
 **/
@Service
public class SysUserInfoService implements ISysUserInfoService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserInfoService.class);

    @Autowired
    private SysUserInfoRepository userInfoRDao;

    @Autowired
    private SysUserRepository userRDao;

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private RefUserDepartmentRepository refUserDepartmentRDao;

    @Autowired
    private PositionRepository positionRDao;

    @Autowired
    private RefUserPositionRepository refUserPositionRDao;

    @Autowired
    private OrganizeRepository organizeRDao;

    /***
     * @param userInfo
     * @return
     */
    @Override
    public SysUserInfo insertSysUserInfo(SysUserInfo userInfo) {
        LOGGER.info("用户信息 SysUserInfo userInfo:{}",userInfo);

        return userInfoRDao.insert(userInfo);
    }

    @Override
    public void updateSysUserInfo(SysUserInfo userInfo) {
        LOGGER.info("用户信息 SysUserInfo userInfo:{}",userInfo);
        userInfoRDao.update(userInfo);
    }

    @Override
    public void refreshRefUserDepartment(String userInfoId, String[] departmentIds, String userName) {
        LOGGER.info("用户信息 SysUserInfo userInfoId:{} departmentIds:{} userName:{}",userInfoId,departmentIds,userName);

        SysUserInfo userInfo = userInfoRDao.getById(Long.parseLong(userInfoId));
        if(userInfo != null ){
            // 查询部门信息
            StringBuffer where = new StringBuffer();
            HqlPack.getInPack(String.join(",",departmentIds),"id",where);
            List<Department> departmentList = departmentRDao.findByHqlWhere(where.toString());

            // 删除用户部门关系
            refUserDepartmentRDao.deleteByProperty("userId",userInfo.getId());

            if(departmentList != null && departmentList.size() >0){
                List<RefUserDepartment> list = new ArrayList<>();
                List<String> nameList = new ArrayList<>();
                RefUserDepartment ref = null;
                for (Department department : departmentList) {
                    ref = new RefUserDepartment();
                    ref.setDepartmentId(department.getId());
                    ref.setDepartmentNameRdd(department.getName());
                    ref.setUserId(userInfo.getId());
                    ref.setUserNameRdd(userInfo.getName());
                    ref.setCreateUser(userName);
                    ref.setCreateDatetime(TimeTool.getNowTime(true));
                    list.add(ref);

                    nameList.add(department.getName());
                }

                userInfo.setDepartmentNameRdd(String.join(",",nameList));

                refUserDepartmentRDao.saveAll(list);
                userInfoRDao.save(userInfo);

                LOGGER.info("用户信息 SysUserInfo RefUserDepartmentRDao Insert Size :{}",list.size());
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefUserPosition(String userInfoId, String[] positionIds, String userName) {
        LOGGER.info("用户信息 SysUserInfo userInfoId:{} positionIds:{} userName:{}",userInfoId,positionIds,userName);

        SysUserInfo userInfo = userInfoRDao.getById(Long.parseLong(userInfoId));
        if(userInfo != null ){
            // 查询部门信息
            StringBuffer where = new StringBuffer();
            HqlPack.getInPack(String.join(",",positionIds),"id",where);
            List<Position> positionList = positionRDao.findByHqlWhere(where.toString());

            // 删除用户岗位关系
            refUserPositionRDao.deleteByProperty("userId",userInfo.getId());

            if(positionList != null && positionList.size() >0){
                List<RefUserPosition> list = new ArrayList<>();
                List<String> nameList = new ArrayList<>();
                RefUserPosition ref = null;
                for (Position position : positionList) {
                    ref = new RefUserPosition();
                    ref.setUserId(userInfo.getId());
                    ref.setUserNameRdd(userInfo.getName());
                    ref.setPositionId(position.getId());
                    ref.setPositionNameRdd(position.getName());
                    ref.setCreateUser(userName);
                    ref.setCreateDatetime(TimeTool.getNowTime(true));
                    list.add(ref);

                    nameList.add(position.getName());
                }

                userInfo.setPositionNameRdd(String.join(",",nameList));

                refUserPositionRDao.saveAll(list);
                userInfoRDao.save(userInfo);

                LOGGER.info("用户信息 SysUserInfo RefUserPositionRDao Insert Size :{}",list.size());
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshRefUserOrganize(String userInfoId, String organizeId, String userName) {
        LOGGER.info("用户信息 SysUserInfo userInfoId:{} organizeId:{} userName:{}",userInfoId,organizeId,userName);
        SysUserInfo userInfo = userInfoRDao.getById(Long.parseLong(userInfoId));
        if(userInfo != null ){
            Organize organize = organizeRDao.getById(Long.parseLong(organizeId));
            if(organize != null ){

                userInfo.setOrganizeId(organize.getId());
                userInfo.setOrganizeNameRdd(organize.getName());
                userInfo.setModifyUser(userName);
                userInfo.setModifyDatetime(TimeTool.getNowTime(true));

                userInfoRDao.update(organize);
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户信息不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void deleteSysUserInfoById(String id) {
        LOGGER.info("用户信息 SysUserInfo id:{}",id);
        userInfoRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void deleteSysUserInfoByIds(String[] ids) {
        LOGGER.info("用户信息 SysUserInfo ids:{}",ids);
        userInfoRDao.deleteByIds(StringTool.getArrayLong(ids));
    }

    @Override
    public SysUserInfo getSysUserInfoById(Long id) {
        return userInfoRDao.getById(id);
    }

    @Override
    public List listSysUserInfo() {
        return userInfoRDao.findAll();
    }

    @Override
    public ListPager<SysUserInfo> querySysUserInfo(SysUserInfo userInfo, Pager pager) {
        LOGGER.info("定时任务 TASK_TIME TaskTime:{}，Pager:{}",userInfo,pager);
        if(userInfo == null) {
            pager = PagerHelper.getPager(pager, userInfoRDao.listCount());
            return new ListPager(userInfoRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysUserInfo(userInfo);
            pager = PagerHelper.getPager(pager, userInfoRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(userInfoRDao.findByHqlWherePage(hqlPack + userInfo.orderBy(),pager),pager);
        }
    }
}
